-------------- MODULE SimpleAllocatorDiagram ---------------------
(****************************************************************)
(* This module adds some definitions for the proof of property  *)
(* ClientsWillObtain of module SimpleAllocator via a predicate  *)
(* diagram.                                                     *)
(****************************************************************)

EXTENDS SimpleAllocator

CONSTANTS c  \** Skolem constant for the client

ASSUME c \in Clients

------------------------------------------------------------------

(** predicates appearing in node labels **)

unsat_c_empty == unsat[c] = {}
unsat_c_finite == IsFiniteSet(unsat[c])
alloc_c_empty == alloc[c] = {}
resource_available == unsat[c] \cap available # {}

(** actions appearing in diagram, other than Next **)

Req_c == \E S \in SUBSET Resources : Request(c,S)

Alloc_c == \E S \in SUBSET Resources : Allocate(c,S)

Return_c == Return(c, alloc[c])

Return_not_c ==
  /\ unsat[c] # {} /\ unsat[c] \cap available = {}
  /\ \E d \in Clients : /\ d#c /\ alloc[d] \cap unsat[c] # {} 
                        /\ Return(d, alloc[d])

(** well-founded ordering relations **)
subset(S,T) == IsFiniteSet(S) /\ IsFiniteSet(T) /\ S \subseteq T /\ S#T
subset_eq(S,T) == IsFiniteSet(S) /\ IsFiniteSet(T) /\ S \subseteq T

==================================================================